{ Copyright (C) 1998-2000, written by Shkolnik Mike
  FIDOnet: 2:463/106.14
  E-Mail:  mshkolnik@scalabium.com
           mshkolnik@yahoo.com
  WEB: http://www.scalabium.com
       http://www.geocities.com/mshkolnik
  tel: 380-/44/-552-10-29

  TSMRuler is a component for Delphi. This ruler can be
  displayed in a horizontal or vertical direction. It has
  properties which allow automatic displaying of current
  position indicator and measurement markers
  (centimeters/inches/pixels). Also TSMRuler has a margins
  (start/end) and offsets (start/end).

}
unit SMRuler;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TSMDirection = (smdHorizontalDown, smdHorizontalUp,
                  smdVerticalLeft, smdVerticalRight);
  TSMMeasure = (smmCentimetre, smmInch, smmPixels);

  TSMRuler = class(TCustomControl {TGraphicControl})
  private
    { Private declarations }
    FPositionColor: TColor;
    FDirection: TSMDirection;
    FMeasure: TSMMeasure;
    FMarginStart: Integer;
    FMarginEnd: Integer;
    FMarginColor: TColor;
    FOffsetStart: Integer;
    FOffsetEnd: Integer;
    FPosition: Integer;
    FPositionStep: Integer;

    procedure SetDirection(Value: TSMDirection);
    procedure SetMeasure(Value: TSMMeasure);
    procedure SetMarginStart(Value: Integer);
    procedure SetMarginEnd(Value: Integer);
    procedure SetMarginColor(Value: TColor);
    procedure SetOffsetStart(Value: Integer);
    procedure SetOffsetEnd(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetPositionColor(Value: TColor);

    procedure UpdatePosition(intPosition: Integer);
    procedure DrawPosition(intPosition: Integer);
  protected
    { Protected declarations }
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property Direction: TSMDirection read FDirection write SetDirection;
    property Measure: TSMMeasure read FMeasure write SetMeasure;
    property MarginStart: Integer read FMarginStart write SetMarginStart;
    property MarginEnd: Integer read FMarginEnd write SetMarginEnd;
    property MarginColor: TColor read FMarginColor write SetMarginColor;
    property Position: Integer read FPosition write SetPosition;
    property PositionColor: TColor read FPositionColor write SetPositionColor;
    property PositionStep: Integer read FPositionStep write FPositionStep;
    property OffsetStart: Integer read FOffsetStart write SetOffsetStart;
    property OffsetEnd: Integer read FOffsetEnd write SetOffsetEnd;

    property Align;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation
{$R *.DCR}

procedure Register;
begin
  RegisterComponents('SMComponents', [TSMRuler]);
end;

{ TSMRuler }
constructor TSMRuler.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FDirection := smdHorizontalUp;
  Align := alTop;
  FMeasure := smmCentimetre;
  FMarginStart := 0;
  FMarginEnd := 0;
  FOffsetStart := 0;
  FOffsetEnd := 0;
  FMarginColor := clBtnShadow;
  FPositionColor := clWhite;
  FPositionStep := 1;
end;

procedure TSMRuler.Paint;
var i, intCoef, intCurrentStep, intStep,
    intCurrentCycle, intCycled,
    X1, X2, Y1, Y2,
    XX1, XX2, YY1, YY2,
    XText, YText, intHeight: Integer;
    ARect: TRect;
    strText: string;
    bmp: TBitmap;
begin
  bmp := TBitmap.Create;
  bmp.Width := Width;
  bmp.Height := Height;
  with bmp.Canvas do
  begin
    Pen.Width := 1;
    Pen.Color := clBlack;
    Brush.Color := Self.Color;
    Brush.Style := bsSolid;
    FillRect(ClientRect);

    if FMarginStart > 0 then
    begin
      Brush.Color := FMarginColor;
      {fill MarginStart}
      case FDirection of
        smdHorizontalUp,
        smdHorizontalDown: ARect := Rect(FOffsetStart, 0, FOffsetStart + FMarginStart, Height);
        smdVerticalLeft,
        smdVerticalRight: ARect := Rect(0, FOffsetStart, Width, FOffsetStart + FMarginStart);
      end;
      FillRect(ARect);
    end;

    if FMarginStart > 0 then
    begin
      Brush.Color := FMarginColor;
      {fill MarginEnd}
      case FDirection of
        smdHorizontalUp,
        smdHorizontalDown: ARect := Rect(Width - FMarginEnd - FOffsetEnd, 0, Width - FOffsetEnd, Height);
        smdVerticalLeft,
        smdVerticalRight: ARect := Rect(0, Height - FMarginEnd - FOffsetEnd, Width, Height - FOffsetEnd);
      end;
      FillRect(ARect);
    end;

    Font := Self.Font;
    case FMeasure of
      smmCentimetre: begin
                       intCycled := 2;
                       intStep := Trunc(Screen.PixelsPerInch/(2.54*intCycled));
                       intCoef := 1;
                     end;
      smmInch: begin
                 intCycled := 3;
                 intStep := Trunc(Screen.PixelsPerInch/intCycled);
                 intCoef := 1;
               end;
      smmPixels: begin
                   intCycled := 2;
                   intStep := Trunc(50/intCycled);
                   intCoef := 50;
                 end;
    end;

    {calculate start coordinates for draw}
    intHeight := TextHeight('999');
    case FDirection of
      smdHorizontalUp: begin
                         X1 := FOffsetStart;
                         X2 := FOffsetStart;
                         Y1 := intHeight + 2;
                         Y2 := Height;

                         XX1 := FOffsetStart;
                         XX2 := FOffsetStart;
                         YY1 := (Y1 + Y2) div 2;
                         YY2 := Height;

                         XText := 0;
                         YText := 0;
                       end;
      smdHorizontalDown: begin
                           X1 := FOffsetStart;
                           X2 := FOffsetStart;
                           Y1 := 0;
                           Y2 := Height - intHeight - 2;

                           XX1 := FOffsetStart;
                           XX2 := FOffsetStart;
                           YY1 := 0;
                           YY2 := (Y1 + Y2) div 2;

                           XText := 0;
                           YText := Height - intHeight;
                         end;
      smdVerticalLeft: begin
                         X1 := TextWidth('999') + 2;
                         X2 := Width;
                         Y1 := FOffsetStart;
                         Y2 := FOffsetStart;

                         XX1 := (X1 + X2) div 2;
                         XX2 := Width;
                         YY1 := FOffsetStart;
                         YY2 := FOffsetStart;

                         XText := 0;
                         YText := 0;
                       end;
      smdVerticalRight: begin
                          X1 := 0;
                          X2 := Width - TextWidth('999') - 2;
                          Y1 := FOffsetStart;
                          Y2 := FOffsetStart;

                          XX1 := 0;
                          XX2 := (X1 + X2) div 2;
                          YY1 := FOffsetStart;
                          YY2 := FOffsetStart;

                          XText := Width - TextWidth('999');
                          YText := 0;
                        end;
    end;

    intCurrentStep := 0;
    intCurrentCycle := 1;
    while (((FDirection = smdHorizontalUp) or (FDirection = smdHorizontalDown)) and (X1 < Width - FOffsetEnd)) or
          (((FDirection = smdVerticalLeft) or (FDirection = smdVerticalRight)) and (Y1 < Height - FOffsetEnd)) do
    begin
      {draw line}
      if (intCurrentCycle = 1) then
      begin
        MoveTo(X1, Y1);
        LineTo(X2, Y2);
      end
      else
      begin
        MoveTo(XX1, YY1);
        LineTo(XX2, YY2);
      end;

      {out text}
      if (intCurrentCycle = 1) then
      begin
        if (intCurrentStep <> 0) then
        begin
          case FDirection of
            smdHorizontalUp,
            smdHorizontalDown: if (XText < FMarginStart) or (XText > Width - FMarginEnd)then
                                 Brush.Color := FMarginColor
                               else
                                 Brush.Color := Self.Color;
            smdVerticalLeft,
            smdVerticalRight: if (YText < FMarginStart) or (YText > Height - FMarginEnd)then
                                 Brush.Color := FMarginColor
                               else
                                 Brush.Color := Self.Color;
          end;

          strText := IntToStr(intCurrentStep*intCoef);
          Brush.Style := bsClear;
          if FDirection in [smdHorizontalUp, smdHorizontalDown] then
            TextOut(XText - (TextWidth(strText) div 2), YText, IntToStr(intCurrentStep*intCoef))
          else
            TextOut(XText, YText - (TextHeight(strText) div 2), IntToStr(intCurrentStep*intCoef));
        end;
        Inc(intCurrentStep);
      end;
      Inc(intCurrentCycle);
      if (intCurrentCycle > intCycled) then
        intCurrentCycle := 1;

      {calculate coordinates for draw}
      case FDirection of
        smdHorizontalUp,
        smdHorizontalDown: begin
                             X1 := X1 + intStep;
                             X2 := X1;
                             XX1 := X1;
                             XX2 := X1;
                             XText := X1;
                           end;
        smdVerticalLeft,
        smdVerticalRight: begin
                            Y1 := Y1 + intStep;
                            Y2 := Y1;
                            YY1 := Y1;
                            YY2 := Y1;
                            YText := Y1;
                          end;
      end;
    end;
  end;
  Canvas.Draw(0, 0, bmp);
  bmp.Free;

  DrawPosition(FPosition);
end;

procedure TSMRuler.DrawPosition(intPosition: Integer);
begin
  {draw current position}
  with Canvas do
  begin
//    Pen.Width := 2;
    Pen.Color := FPositionColor;
    case FDirection of
      smdHorizontalUp,
      smdHorizontalDown: begin
                           MoveTo(intPosition, 0);
                           LineTo(intPosition, Height);
                         end;
      smdVerticalLeft,
      smdVerticalRight: begin
                          MoveTo(0, intPosition);
                          LineTo(Width, intPosition);
                        end;
    end;
  end;
end;

procedure TSMRuler.SetDirection(Value: TSMDirection);
begin
  if (Value <> FDirection) then
  begin
    FDirection := Value;
    case Value of
      smdHorizontalUp: Align := alTop;
      smdHorizontalDown: Align := alBottom;
      smdVerticalLeft: Align := alLeft;
      smdVerticalRight: Align := alRight
    end;

    Invalidate
  end;
end;

procedure TSMRuler.SetMeasure(Value: TSMMeasure);
begin
  if (Value <> FMeasure) then
  begin
    FMeasure := Value;
    Invalidate
  end;
end;

procedure TSMRuler.SetMarginStart(Value: Integer);
begin
  if (Value <> FMarginStart) then
  begin
    FMarginStart := Value;
    Invalidate
  end;
end;

procedure TSMRuler.SetMarginEnd(Value: Integer);
begin
  if (Value <> FMarginEnd) then
  begin
    FMarginEnd := Value;
    Invalidate
  end;
end;

procedure TSMRuler.SetMarginColor(Value: TColor);
begin
  if (Value <> FMarginColor) then
  begin
    FMarginColor := Value;
    Invalidate
  end;
end;

procedure TSMRuler.SetOffsetStart(Value: Integer);
begin
  if (Value <> FOffsetStart) then
  begin
    FOffsetStart := Value;
    Invalidate
  end;
end;

procedure TSMRuler.SetOffsetEnd(Value: Integer);
begin
  if (Value <> FOffsetEnd) then
  begin
    FOffsetEnd := Value;
    Invalidate
  end;
end;

procedure TSMRuler.SetPosition(Value: Integer);
begin
  if (Abs(Value - FPosition) >= FPositionStep) then
  begin
    UpdatePosition(Value);
    FPosition := Value;
  end;
end;

procedure TSMRuler.UpdatePosition(intPosition: Integer);
var oldRect, newRect: TRect;
begin
  if Direction in [smdVerticalLeft, smdVerticalRight] then
  begin
    oldRect := Bounds(0, FPosition, Width, 1);
    newRect := Bounds(0, intPosition, Width, 1);
  end
  else
  begin
    oldRect := Bounds(FPosition, 0, 1, Height);
    newRect := Bounds(intPosition, 0, 1, Height);
  end;

  {replace last position to avoid flickering}
  InvalidateRect(Handle, @oldRect, False);

  {draw a new current position}
  DrawPosition(intPosition);
end;

procedure TSMRuler.SetPositionColor(Value: TColor);
begin
  if (Value <> FPositionColor) then
  begin
    FPositionColor := Value;
    Invalidate
  end;
end;

end.
